@@ -2,25 +2,32 @@
 # Copyright 2010 Canonical Ltd.  This software is licensed under the
 # GNU Lesser General Public License version 3 (see the file LICENSE).
 
+import inspect
 import textwrap
 from cStringIO import StringIO
 from unittest import TestCase
 
 import django
 from configglue.pyschema.schema import (
-    DictConfigOption,
-    IntConfigOption,
+    BoolOption,
+    DictOption,
+    IntOption,
+    ListOption,
     Schema,
-    StringConfigOption,
+    Section,
+    StringOption,
+    TupleOption,
 )
 from configglue.pyschema.parser import (
     CONFIG_FILE_ENCODING,
     SchemaConfigParser,
 )
 from django.conf import settings
+from django.conf import global_settings
+from django.conf.project_template import settings as project_settings
 from mock import patch
 
-from django_configglue import GlueManagementUtility
+from django_configglue.management import GlueManagementUtility
 from django_configglue.utils import (
     SETTINGS_ENCODING,
     configglue,
@@ -30,19 +37,23 @@ from django_configglue.utils import (
 from django_configglue.schema import (
     BaseDjangoSchema,
     DjangoSchemaFactory,
-    UpperCaseDictConfigOption,
+    UpperCaseDictOption,
     schemas,
+    derivate_django_schema,
+)
+from django_configglue.tests.helpers import (
+    ConfigGlueDjangoCommandTestCase,
+    SchemaHelperTestCase,
 )
-from django_configglue.tests.helpers import ConfigGlueDjangoCommandTestCase
 
 
-class DjangoSupportTestCase(TestCase):
+class DjangoSupportTestCase(SchemaHelperTestCase):
     def test_get_django_settings(self):
         class MySchema(Schema):
-            foo = IntConfigOption()
-            bar = DictConfigOption(
-                spec={'baz': IntConfigOption(),
-                      'BAZ': IntConfigOption()})
+            foo = IntOption()
+            bar = DictOption(
+                spec={'baz': IntOption(),
+                      'BAZ': IntOption()})
 
         expected = {'FOO': 0, 'BAR': {'baz': 0, 'BAZ': 0}}
 
@@ -52,7 +63,7 @@ class DjangoSupportTestCase(TestCase):
 
     def test_get_django_settings_encoding(self):
         class MySchema(Schema):
-            foo = StringConfigOption()
+            foo = StringOption()
 
         expected = {'FOO': u'€'.encode(SETTINGS_ENCODING)}
 
@@ -65,7 +76,7 @@ class DjangoSupportTestCase(TestCase):
 
     def test_update_settings(self):
         class MySchema(Schema):
-            foo = IntConfigOption()
+            foo = IntOption()
 
         env = {}
         parser = SchemaConfigParser(MySchema())
@@ -79,62 +90,11 @@ class DjangoSupportTestCase(TestCase):
 
     def test_schemafactory_get(self):
         # test get valid version
-        self.assertEqual(schemas.get('1.0.2 final'), BaseDjangoSchema)
+        self.assertEqual(schemas.get('1.0.2 final', strict=True),
+            BaseDjangoSchema)
 
         # test get invalid version
-        self.assertRaises(ValueError, schemas.get, '1.1')
-
-    @patch('django_configglue.schema.logging')
-    def test_schemafactory_get_nonexisting_too_old(self, mock_logging):
-        schema = schemas.get('0.96', strict=False)
-
-        django_102 = schemas.get('1.0.2 final')
-        self.assertEqual(schema, django_102)
-        self.assertRaises(ValueError, schemas.get, '0.96')
-
-        self.assertEqual(mock_logging.warn.call_args_list[0][0][0],
-            "No schema registered for version '0.96'")
-        self.assertEqual(mock_logging.warn.call_args_list[1][0][0],
-            "Falling back to schema for version '1.0.2 final'")
-
-    @patch('django_configglue.schema.logging')
-    def test_schemafactory_get_nonexisting(self, mock_logging):
-        schema = schemas.get('1.0.3', strict=False)
-
-        django_102 = schemas.get('1.0.2 final')
-        self.assertEqual(schema, django_102)
-        self.assertRaises(ValueError, schemas.get, '1.0.3')
-
-        self.assertEqual(mock_logging.warn.call_args_list[0][0][0],
-            "No schema registered for version '1.0.3'")
-        self.assertEqual(mock_logging.warn.call_args_list[1][0][0],
-            "Falling back to schema for version '1.0.2 final'")
-
-    @patch('django_configglue.schema.logging')
-    def test_schemafactory_get_nonexisting_too_new(self, mock_logging):
-        schema = schemas.get('1.2.0', strict=False)
-
-        django_112 = schemas.get('1.1.4')
-        self.assertEqual(schema, django_112)
-        self.assertRaises(ValueError, schemas.get, '1.2.0')
-
-        self.assertEqual(mock_logging.warn.call_args_list[0][0][0],
-            "No schema registered for version '1.2.0'")
-        self.assertEqual(mock_logging.warn.call_args_list[1][0][0],
-            "Falling back to schema for version '1.1.4'")
-
-    @patch('django_configglue.schema.logging')
-    def test_schemafactory_get_no_versions_registered(self, mock_logging):
-        schemas = DjangoSchemaFactory()
-        try:
-            schemas.get('1.0.2 final', strict=False)
-        except ValueError, e:
-            self.assertEqual(str(e), "No schemas registered")
-        else:
-            self.fail("ValueError not raised")
-
-        mock_logging.warn.assert_called_with(
-            "No schema registered for version '1.0.2 final'")
+        self.assertRaises(ValueError, schemas.get, '1.1', strict=True)
 
     def test_schema_versions(self):
         django_102 = schemas.get('1.0.2 final')()
@@ -176,6 +136,133 @@ class DjangoSupportTestCase(TestCase):
         mock_update_settings.assert_called_with(
             MockSchemaConfigParser.return_value, target)
 
+    def test_schemafactory_build(self):
+        schemas = DjangoSchemaFactory()
+
+        data = [
+            (IntOption, 1),
+            (BoolOption, True),
+            (TupleOption, (1, 2)),
+            (ListOption, [1, 2]),
+            (DictOption, {'a': 1}),
+            (StringOption, 'foo'),
+        ]
+
+        for opt_type, default in data:
+            schema_cls = schemas.build('bogus', {'foo': default})
+            # do common checks
+            self.assert_schema_structure(schema_cls, 'bogus',
+                {'foo': opt_type(name='foo', default=default)})
+
+    def test_schemafactory_build_django(self):
+        schemas = DjangoSchemaFactory()
+
+        schema = schemas.build()
+        # get known django schema
+        # fail for unknown versions of django
+        django_schema = schemas.get(django.get_version(), strict=True)
+
+        self.assert_schemas_equal(schema(), django_schema())
+
+    def test_schemafactory_get_django(self):
+        schemas = DjangoSchemaFactory()
+
+        # build a django schema
+        # get only django settings
+        options = dict([(name, value) for (name, value) in
+            inspect.getmembers(settings) if name.isupper()])
+        schema = schemas.build(django.get_version(), options)
+        # get known django schema
+        # fail for unknown versions of django
+        django_schema = schemas.get(django.get_version(), strict=True)
+
+        self.assert_schemas_equal(schema(), django_schema())
+
+    def test_schemafactory_build_complex_options(self):
+        schemas = DjangoSchemaFactory()
+
+        options = {
+            'dict1': {'foo': 1, 'bar': '2'},
+            'dict2': {'foo': {'bar': 2, 'baz': 3}},
+            'list1': ['1', '2', '3'],
+            'list2': [1, 2, 3],
+        }
+        expected = {
+            'dict1': DictOption(name='dict1', default=options['dict1'],
+                                item=StringOption()),
+            'dict2': DictOption(name='dict2', default=options['dict2'],
+                                item=DictOption()),
+            'list1': ListOption(name='list1', default=options['list1'],
+                                item=StringOption()),
+            'list2': ListOption(name='list2', default=options['list2'],
+                                item=IntOption()),
+        }
+
+        schema = schemas.build('foo', options)()
+
+        sections = sorted(
+            [section.name for section in schema.sections()])
+        section = schema.section('django')
+        self.assertEqual(sections, ['django'])
+
+        for name in options:
+            option = expected[name]
+            option.section = section
+            self.assertEqual(section.option(name), option)
+            self.assertEqual(section.option(name).item, option.item)
+
+    @patch('django_configglue.schema.logging')
+    def test_schemafactory_get_unknown_build(self, mock_logging):
+        schemas = DjangoSchemaFactory()
+        version_string = django.get_version()
+        self.assertFalse(version_string in schemas._schemas)
+
+        # build schema
+        schema = schemas.get(version_string, strict=False)()
+        # get expected schema
+        options = dict([(name.lower(), value) for (name, value) in
+            inspect.getmembers(global_settings) if name.isupper()])
+        project_options = dict([(name.lower(), value) for (name, value) in
+            inspect.getmembers(project_settings) if name.isupper()])
+        options.update(project_options)
+        expected = schemas.build(django.get_version(), options)()
+
+        # compare schemas
+        self.assert_schemas_equal(schema, expected)
+
+        self.assertEqual(mock_logging.warn.call_args_list[0][0][0],
+            "No schema registered for version '{0}'".format(version_string))
+        self.assertEqual(mock_logging.warn.call_args_list[1][0][0],
+            "Dynamically creating schema for version '{0}'".format(
+                version_string))
+
+    def test_derivate_django_schema(self):
+        class Parent(Schema):
+            version = 'parent'
+
+            class django(Section):
+                foo = IntOption()
+                bar = IntOption()
+
+        class Child(Parent):
+            version = 'parent'
+
+            class django(Section):
+                bar = IntOption()
+
+        derivated = derivate_django_schema(Parent, exclude=['foo'])
+        self.assert_schemas_equal(derivated(), Child())
+
+    def test_derivate_django_schema_no_exclude(self):
+        class Parent(Schema):
+            version = 'parent'
+
+            class django(Section):
+                foo = IntOption()
+
+        derivated = derivate_django_schema(Parent)
+        self.assertEqual(derivated, Parent)
+
 
 class GlueManagementUtilityTestCase(ConfigGlueDjangoCommandTestCase):
     def setUp(self):
@@ -231,7 +318,8 @@ class GlueManagementUtilityTestCase(ConfigGlueDjangoCommandTestCase):
         try:
             self.util.argv = ['', 'help']
             self.assertRaises(SystemExit, self.execute)
-            self.assertTrue(self.util.main_help_text() in self.capture['stderr'])
+            self.assertTrue(
+                self.util.main_help_text() in self.capture['stderr'])
         finally:
             wrapped.__CONFIGGLUE_PARSER__ = old_CONFIGGLUE_PARSER
 
@@ -249,10 +337,10 @@ class GlueManagementUtilityTestCase(ConfigGlueDjangoCommandTestCase):
         self.assertTrue('Show settings attributes' in self.capture['stdout'])
 
 
-class UpperCaseDictConfigOptionTestCase(TestCase):
+class UpperCaseDictOptionTestCase(TestCase):
     def test_parse(self):
         class MySchema(Schema):
-            foo = UpperCaseDictConfigOption()
+            foo = UpperCaseDictOption()
         config = StringIO(textwrap.dedent("""
             [__main__]
             foo = mydict
@@ -266,4 +354,3 @@ class UpperCaseDictConfigOptionTestCase(TestCase):
         result = schema.foo.parse('mydict', parser)
 
         self.assertEqual(result, {'BAR': '42'})
-
